@@ -11,16 +11,16 @@ function(x, fun, ..., filename="", overwrite=FALSE, wopt=list())  {
 	v <- readValues(x, round(0.5*nrow(x)), 1, 1, nc, mat=TRUE)
 	r <- try(fun(as.vector(v), ...))
 	if (inherits(r, "try-error")) {
-		stop("'fun' is not valid")
+		error("arith", "'fun' is not valid")
 	}
 	if (!is.vector(r)) {
-		stop("'fun' does not return a vector")
+		error("arith", "'fun' does not return a vector")
 	}
 	if (!(is.numeric(r) | is.logical(r))) {
-		stop("'fun' does not return a numeric vector")
+		error("arith", "'fun' does not return a numeric vector")
 	} 
 	if (length(r) != length(v)) {
-		stop("'fun' does not return the same number of values as the input")
+		error("arith", "'fun' does not return the same number of values as the input")
 	}
 
 	b <- writeStart(out, filename, overwrite, wopt)
