@@ -27,7 +27,7 @@
 	cells <- cellFromXY(x, xyCoords)
 	cells <- unique(stats::na.omit(cells))
 	if (length(cells) == 0 ) { 
-		stop('no valid cells selected') 
+		error("click", "no valid cells selected") 
 	}
 	cells
 }
@@ -41,7 +41,7 @@ setMethod("click", signature(x="missing"),
 	}
 )
 
-	
+
 setMethod("click", signature(x="SpatRaster"), 
 	function(x, n=Inf, id=FALSE, xy=FALSE, cell=FALSE, type="n", show=TRUE, ...) {
 	values <- NULL
@@ -57,7 +57,7 @@ setMethod("click", signature(x="SpatRaster"),
 		}
 		cells <- stats::na.omit(cellFromXY(x, xyCoords))
 		if (length(cells) == 0) break
-		
+
 		value <- x[cells]
 		if (cell) {
 			value <- data.frame(cell=cells, value)
@@ -86,4 +86,13 @@ setMethod("click", signature(x="SpatRaster"),
 
 
 
+setMethod("click", signature(x="SpatVector"), 
+	function(x, n=1, type="n", ...) {
+		loc <- graphics::locator(n, type, ...)
+		xy <- vect(cbind(x=loc$x, y=loc$y))
+		e <- extract(xy, x)
+		e[,-1]
+	}
+)
+
 
