@@ -10,24 +10,26 @@
 setMethod("Math", signature(x="SpatRaster"),
     function(x){ 
 		oper <- as.vector(.Generic)[1]
+		opt <- spatOptions("", TRUE, list())
 		if (substr(oper, 1, 3) == "cum") {
 			x@ptr <- x@ptr$cum(substr(oper, 4, 10), FALSE, "", FALSE)
 		} else if (oper %in% c("acos", "acosh", "asin", "asinh", "atan", "atanh", "cos", "cosh", "cospi", "sin", "sinh", "sinpi", "tan", "tanh", "tanpi")) {
-			x@ptr <- x@ptr$trig(oper, .terra_environment$options@ptr)
+			x@ptr <- x@ptr$trig(oper, opt)
 		} else {
-			x@ptr <- x@ptr$math(oper, .terra_environment$options@ptr)
+			x@ptr <- x@ptr$math(oper, opt)
 		}
-		show_messages(x, oper)
-	}	
+		messages(x, oper)
+	}
 )
 
 
 setMethod("Math2", signature(x="SpatRaster"),
     function(x, digits=0){ 
+		opt <- spatOptions("", TRUE, list())
 		oper <- as.vector(.Generic)[1]
-		x@ptr <- x@ptr$math2(oper, digits, .terra_environment$options@ptr)
-		show_messages(x, oper)
-	}	
+		x@ptr <- x@ptr$math2(oper, digits, opt)
+		messages(x, oper)
+	}
 )
 
 
@@ -39,13 +41,13 @@ setMethod("Math", signature(x="SpatExtent"),
 		} else if (oper == "ceiling") {
 			x@ptr <- x@ptr$ceil()
 		} else {
-			stop("not implemented for SpatExtent")
+			error(oper, "not implemented for SpatExtent")
 		}
 		if (!x@ptr$valid) {
-			stop("invalid extent")
+			error(oper, "invalid extent")
 		}
-		return(x)		
-	}	
+		return(x)
+	}
 )
 
 setMethod("Math2", signature(x="SpatExtent"),
@@ -54,13 +56,13 @@ setMethod("Math2", signature(x="SpatExtent"),
 		if (oper == "round") {
 			x@ptr <- x@ptr$round(digits)
 			if (!x@ptr$valid) {
-				stop("invalid extent")
+				error(oper, "invalid extent")
 			}
 			return(x)
 		} else {
-			stop("not implemented for SpatExtent")
+			error(oper, "not implemented for SpatExtent")
 		}
-	}	
+	}
 )
 
 
