@@ -10,13 +10,13 @@
 }
  
 .options_names <- function() {
-	c("progress", "tempdir", "memfrac", "datatype", "filetype", "filenames", "overwrite", "todisk", "names", "verbose", "NAflag") 
+	c("progress", "tempdir", "memfrac", "datatype", "filetype", "filenames", "overwrite", "todisk", "names", "verbose", "NAflag", "statistics") 
 }
 
  
 .setOptions <- function(x, opt) {
 	nms <- names(opt)
-	
+
 	g <- which(nms == "gdal")
 	if (length(g) > 0) {
 		gopt <- unlist(opt[g])
@@ -27,43 +27,47 @@
 		gopt <- gsub(" ", "", gopt)
 		x$gdal_options <- gopt
 	}
-	
+
 	s <- nms %in% .options_names()
-	
+
 	if (any(!s)) {
 		bad <- paste(nms[!s], collapse=",")
-		warning(paste("unknown write option(s):", bad), call. = FALSE)
+		error("write", "unknown option(s):", bad)
 	}
-		
+
 	if (any(s)) {
 		nms <- nms[s]
 		opt <- opt[s]
-		i <- which(nms == "names")	
+		i <- which(nms == "names")
 		if (length(i) > 0) {
 			namevs <- trimws(unlist(strsplit(opt[[i]], ",")))
 			x[["names"]] <- namevs
 			opt <- opt[-i]
 			nms <- nms[-i]
 		}
-		
+
 		for (i in seq_along(nms)) {
 			x[[nms[i]]] <- opt[[i]]
 		}
+		if ("datatype" %in% nms) {
+			x$datatype_set = TRUE;
+		}
 	}
+
 	x
 } 
  
-.runOptions <- function(filename="", overwrite=FALSE, wopt=list()) {
+spatOptions <- function(filename="", overwrite=FALSE, wopt=list()) {
 	if (!is.list(wopt)) {
-		stop("wopt must be a list")
+		error("spatOptions", "wopt must be a list")
 	}
-	
+
 	## work around onLoad problem
 	if (is.null(.terra_environment$options)) .create_options()
-	
+
 	ptr <- .terra_environment$options@ptr
 	opt <- ptr$deepcopy(ptr)
-	
+
 	filename <- .fullFilename(filename, mustExist=FALSE)
 	if (!is.null(unlist(wopt))) {
 		wopt$filenames <- filename
@@ -73,13 +77,13 @@
 		opt$filenames <- filename
 		opt$overwrite <- overwrite[1]
 	}
-	#show_messages(opt)
+	#messages(opt)
 	#opt$todisk <- TRUE
 	opt
 }
 
 .getOptions <- function() {
-	.runOptions("", TRUE, list())
+	spatOptions("", TRUE, list())
 }
 
 ..showOptions <- function(opt) {
@@ -102,6 +106,13 @@
 		cat(paste0(substr(paste(n, "         "), 1, 10), ": ", v, "\n"))
 	}
 }
+
+
+.default_option_names <- function() {
+	c("datatype", "filetype", "verbose") 
+}
+
+
  
 terraOptions <- function(...) {
 	dots <- list(...)
@@ -109,7 +120,16 @@ terraOptions <- function(...) {
 	if (length(dots) == 0) {
 		.showOptions(opt)
 	} else {
-		opt <- .setOptions(opt, dots)			
+		nms <- names(dots)
+		d <- nms %in% .default_option_names()
+		dnms <- paste0("def_", nms)
+		for (i in 1:length(nms)) {
+			if (d[i]) {
+				opt[[ dnms[i] ]] <- dots[[ i ]]
+			} else {
+				opt[[ nms[i] ]] <- dots[[ i ]]
+			}
+		}
 		.terra_environment$options@ptr <- opt
 	}
 }
