@@ -7,8 +7,8 @@ setClass("PackedSpatVector",
 		coordinates = "matrix",
 		index = "matrix",
 		attributes = "data.frame"
-	),	
-	prototype (	
+	),
+	prototype (
 		type= "",
 		crs = ""
 	)
@@ -20,7 +20,7 @@ setClass("PackedSpatRaster",
 		definition = "character",
 		values = "matrix",
 		attributes = "data.frame"
-	),	
+	),
 )
 
 
@@ -29,7 +29,7 @@ setClass("PackedSpatRaster",
 	vd@type <- geomtype(x)
 	vd@crs <- as.character(crs(x))
 	stopifnot(vd@type %in% c("points", "lines", "polygons"))
-	g <- as.matrix(geom(x))
+	g <- geom(x)
 	vd@coordinates <- g[, c("x", "y")]
 	j <- c(1,2, grep("hole", colnames(g)))
 	g <- g[,j]
@@ -42,7 +42,7 @@ setMethod("pack", signature(x="Spatial"),
 	function(x, ...) {
 		pv <- .packVector(x)
 		if (methods::.hasSlot(x, "data")) {
-			pv@attributes <- x@data	
+			pv@attributes <- x@data
 		}
 		pv
 	}
@@ -73,20 +73,20 @@ setMethod("vect", signature(x="PackedSpatVector"),
 		reps <- diff(c(x@index[,n], nrow(x@coordinates)+1))
 		i <- rep(1:nrow(x@index), reps)
 		if (n == 2) { 
-			p@ptr$setGeometry(x@type, x@index[i,1], x@index[i,2], x@coordinates[,1], x@coordinates[,2], rep(0, nrow(x@coordinates)))		
+			p@ptr$setGeometry(x@type, x@index[i,1], x@index[i,2], x@coordinates[,1], x@coordinates[,2], rep(0, nrow(x@coordinates)))
 		} else {
 			p@ptr$setGeometry(x@type, x@index[i,1], x@index[i,2], x@coordinates[,1], x@coordinates[,2], x@index[i,3])
 		} 
 		if (nrow(x@attributes) > 0) {
 			values(p) <- x@attributes
 		}
-		show_messages(p, "pack")
+		messages(p, "pack")
 	}
 )
 
 setMethod("show", signature(object="PackedSpatVector"), 
 	function(object) {
-		print(paste("This is a", class(object), "object. Use 'vect()' to unpack it"))
+		print(paste("This is a", class(object), "object. Use 'terra::vect()' to unpack it"))
 	}
 )
 
@@ -137,6 +137,6 @@ setMethod("rast", signature(x="PackedSpatRaster"),
 
 setMethod("show", signature(object="PackedSpatRaster"), 
 	function(object) {
-		print(paste("This is a", class(object), "object. Use 'rast()' to unpack it"))
+		print(paste("This is a", class(object), "object. Use 'terra::rast()' to unpack it"))
 	}
 )
