@@ -5,12 +5,17 @@
 
 
 setMethod("rast", signature(x="missing"),
-	function(x, nrows=180, ncols=360, nlyrs=1, xmin=-180, xmax=180, ymin=-90, ymax=90, crs, extent, resolution, ...) {
+	function(x, nrows=180, ncols=360, nlyrs=1, xmin=-180, xmax=180, ymin=-90, ymax=90, crs, extent, resolution, vals) {
+
+		if (missing(extent)) {
+			e <- c(xmin, xmax, ymin, ymax) 
+		} else {
+			e <- as.vector(extent)
+		}
+		if ((e[1] >= e[2]) || e[3] >= e[4]) {
+			error("rast,missing", "invalid extent")
+		}
 
-		if (missing(extent)) {	extent <- ext(xmin, xmax, ymin, ymax) }
-		e <- as.vector(extent)
-		if ((e[1] >= e[2]) || e[3] >= e[4]) {stop("invalid extent")}
-		
 		if (missing(crs)) {
 			if (e[1] > -360.01 & e[2] < 360.01 & e[3] > -90.01 & e[4] < 90.01) {
 				crs <- "+proj=longlat +datum=WGS84"
@@ -22,26 +27,33 @@ setMethod("rast", signature(x="missing"),
 			crs <- as.character(crs)
 		}
 
-		if (!missing(resolution)) {
-			nrows <- max(1, round((e[4] - e[3]) / resolution))
-			ncols <- max(1, round((e[2] - e[1]) / resolution))
-		}
-
 		r <- methods::new("SpatRaster")
 		r@ptr <- SpatRaster$new(c(nrows, ncols, nlyrs), e, crs)
+		r <- messages(r, "rast")
 
-		show_messages(r, "rast")
+		if (!missing(resolution)) {
+			res(r) <- resolution
+		}
+
+		if (!missing(vals)) {
+			if (length(vals) == ncell(r)) {
+				values(r) <- vals
+			} else {
+				values(r) <- rep_len(vals, ncell(r))
+			}
+		}
+		r
 	}
 )
 
 setMethod("rast", signature(x="list"),
-	function(x, ...) {
+	function(x) {
 		i <- sapply(x, function(i) inherits(i, "SpatRaster"))
 		if (!any(i)) {
-			stop("None of the elements of x are a SpatRaster")
+			error("rast,list", "none of the elements of x are a SpatRaster")
 		}
 		if (!all(i)) {
-			warning(paste(sum(!i), "out of", length(x), "elements of x are a SpatRaster"))
+			warn("rast", sum(!i), " out of ", length(x), " elements of x are a SpatRaster")
 		}
 		names(x) <- NULL
 		do.call(c, x[i])
@@ -51,18 +63,34 @@ setMethod("rast", signature(x="list"),
 
 setMethod("rast", signature(x="SpatExtent"),
 	function(x, ...) {
-		e <- as.vector(x)
-		rast(xmin=e[1], xmax=e[2], ymin=e[3], ymax=e[4], ...)
+		dots <- list(...)
+		dots$xmin=x[1]
+		dots$xmax=x[2]
+		dots$ymin=x[3]
+		dots$ymax=x[4]
+		if (all(is.na(pmatch(names(dots), "resolution")))) {
+			dots$resolution <- min(range(x)) / 100
+		}
+		do.call(rast, dots)
 	}
 )
 
+
 setMethod("rast", signature(x="SpatVector"),
 	function(x, ...) {
+		dots <- list(...)
 		e <- ext(x)
-		dx <- diff(as.vector(e)[1:2])
-		dy <- diff(as.vector(e)[3:4])
-		res <- min(dx, dy) / 100
-		rast(ext(x), res=res, ...)
+		dots$xmin=e[1]
+		dots$xmax=e[2]
+		dots$ymin=e[3]
+		dots$ymax=e[4]
+		if (all(is.na(pmatch(names(dots), "resolution")))) {
+			dots$resolution <- min(range(e)) / 100
+		}
+		if (all(is.na(pmatch(names(dots), "crs")))) {
+			dots$crs <- crs(x)
+		}
+		do.call(rast, dots)
 	}
 )
 
@@ -91,33 +119,31 @@ setMethod("rast", signature(x="SpatVector"),
 }
 
 setMethod("rast", signature(x="character"),
-	function(x, subds=0, ...) {
-		## work around onLoad problem
-		#if (is.null(.terra_environment$options)) .init()
+	function(x, subds=0) {
 
 		x <- trimws(x)
 		x <- x[x!=""]
 		if (length(x) == 0) {
-			stop("provide a valid filename")
+			error("rast,character", "provide a valid filename")
 		}
 		r <- methods::new("SpatRaster")
 		f <- .fullFilename(x)
 		#subds <- subds[1]
 		if (is.character(subds)) { 
-			r@ptr <- SpatRaster$new(f, -1, subds, "")		
+			r@ptr <- SpatRaster$new(f, -1, subds, "")
 		} else {
 			r@ptr <- terra:::SpatRaster$new(f, subds-1, "", "")
 		}
-		if (r@ptr$messages$getMessage() == "ncdf extent") {
+		if (r@ptr$getMessage() == "ncdf extent") {
 			test <- try(r <- .ncdf_extent(r), silent=TRUE)
 			if (inherits(test, "try-error")) {
-				warning("GDAL did not find an extent. Cells not equally spaced?") 
+				warn("rast", "GDAL did not find an extent. Cells not equally spaced?") 
 			}
 		}
-		r <- show_messages(r, "rast")
-		
+		r <- messages(r, "rast")
+
 		if (crs(r) == "") {
-			if (isLonLat(r, perhaps=TRUE, warn=FALSE)) {
+			if (is.lonlat(r, perhaps=TRUE, warn=FALSE)) {
 				crs(r) <- "+proj=longlat +datum=WGS84"
 			}
 		}
@@ -127,20 +153,16 @@ setMethod("rast", signature(x="character"),
 
 
 setMethod("rast", signature(x="SpatRaster"),
-	function(x, nlyrs=nlyr(x), ...) {
-		r <- methods::new("SpatRaster")
-		r@ptr <- x@ptr$geometry(nlyrs, FALSE)
-		if (length(list(...)) > 0) {
-			warning("additional arguments are ignored")
-		}
-		show_messages(r, "rast")
+	function(x, nlyrs=nlyr(x)) {
+		x@ptr <- x@ptr$geometry(nlyrs, FALSE)
+		messages(x, "rast")
 	}
 )
 
 
-setMethod("rast", signature(x="SpatDataSet"),
-	function(x, ...) {
-		rast(x[1], ...)
+setMethod("rast", signature(x="SpatRasterDataset"),
+	function(x) {
+		rast(x[1])
 	}
 )
 
@@ -150,18 +172,18 @@ setMethod("rast", signature(x="array"),
 	function(x, ...) {
 		dims <- dim(x)
 		if (length(dims) > 3) {
-			stop("cannot handle an array with more than 3 dimensions")
+			error("rast,array", "cannot handle an array with more than 3 dimensions")
 		}
 		r <- methods::new("SpatRaster")
 		r@ptr <- SpatRaster$new(dims, c(0, dims[2], 0, dims[1]), "")
 		values(r) <- x
-		show_messages(r, "rast")
+		messages(r, "rast")
 	}
 )
 
 
 
-setMethod("rast", signature(x="Raster"),
+setMethod("rast", signature(x="ANY"),
 	function(x, ...) {
 		methods::as(x, "SpatRaster")
 	}
@@ -170,7 +192,7 @@ setMethod("rast", signature(x="Raster"),
 
 .rastFromXYZ <- function(xyz, digits=6, crs="", ...) {
 
-	if (length(list(...))>0) warning("additional arguments ignored when x is a SpatDataSet")
+	if (length(list(...))>0) warn("rast (xyz)", "additional arguments ignored when x is a SpatRasterDataset")
 
 	ln <- colnames(xyz)
 	## xyz might not have colnames, or might have "" names
@@ -191,7 +213,7 @@ setMethod("rast", signature(x="Raster"),
 		}
 	}
 	if ( q > 0 ) {
-		stop("x cell sizes are not regular")
+		error("raster,matrix(xyz)", "x cell sizes are not regular")
 	}
 
 	y <- sort(unique(xyz[,2]))
@@ -209,7 +231,7 @@ setMethod("rast", signature(x="Raster"),
 		}
 	}
 	if ( q > 0 ) {
-		stop("y cell sizes are not regular")
+		error("raster,matrix(xyz)", "y cell sizes are not regular")
 	}
 
 	minx <- min(x) - 0.5 * rx
@@ -223,8 +245,8 @@ setMethod("rast", signature(x="Raster"),
 	cells <- cellFromXY(r, xyz[,1:2])
 	if (d[2] > 2) {
 		names(r) <- ln[-c(1:2)]
-		v <- rep(NA, ncell(r))
-		v[cells] <- xyz[,3:d[2]]
+		v <- matrix(NA, nrow=ncell(r), ncol= nlyr(r))
+		v[cells, ] <- xyz[, -c(1:2)]
 		values(r) <- v
 	}
 	return(r)
@@ -232,15 +254,33 @@ setMethod("rast", signature(x="Raster"),
 
 
 
+
 setMethod("rast", signature(x="matrix"),
-	function(x, type="", crs="", ...) {
+	function(x, type="", crs="") {
 		if (type == "xyz") {
-			r <- .rastFromXYZ(x, ...)
+			r <- .rastFromXYZ(x, crs=crs)
 		} else {
-			r <- rast(nrows=nrow(x), ncols=ncol(x), extent=ext(c(0, ncol(x), 0, nrow(x))), ...)
+			r <- rast(nrow=nrow(x), ncol=ncol(x), crs=crs, extent=ext(c(0, 1, 0, 1)))
 			values(r) <- t(x)
 		}
-		show_messages(r, "rast")
+		messages(r, "rast")
+	}
+)
+
+
+setMethod("NAflag<-", signature(x="SpatRaster"), 
+	function(x, ..., value)  {
+		value <- as.numeric(value)
+		if (!(x@ptr$setNAflag(value))) {
+			error("NAflag<-", "cannot set this value")
+		}
+		x
+	}
+)
+
+setMethod("NAflag", signature(x="SpatRaster"), 
+	function(x, ...)  {
+		x@ptr$getNAflag()
 	}
 )
 
