@@ -6,6 +6,9 @@
 
 setMethod("time", signature(x="SpatRaster"), 
 	function(x, ...) { 
+		if (!x@ptr$hasTime) {
+			return(rep(NA, nlyr(x)))
+		}
 		d <- x@ptr$time
 		tstep <- x@ptr$timestep 
 		if (tstep == "seconds") {
@@ -13,6 +16,7 @@ setMethod("time", signature(x="SpatRaster"),
 		} else if (tstep == "days") {
 			as.Date("1970-01-01") + d
 		#} else if (tstep == "months") {
+		#} else if (tstep == "years") {
 		} else { # raw 
 			d
 		}
@@ -27,12 +31,12 @@ setMethod("time<-", signature(x="SpatRaster"),
 		}
 		if (inherits(value, "POSIXt")) {
 			if (!x@ptr$setTime(as.numeric(value), "seconds")) {
-				stop("cannot set these  values")
+				error("time<-", "cannot set these  values")
 			}
 		} else {
 			if (!x@ptr$setTime(as.numeric(value), "raw")) {
-				stop("cannot set these  values")
-			}		
+				error("time<-", "cannot set these  values")
+			}
 		}
 		return(x)
 	}
@@ -51,12 +55,23 @@ setMethod("depth<-", signature(x="SpatRaster"),
 	function(x, value)  {
 		value <- as.numeric(value)
 		if (! x@ptr$setDepth(value)) {
-			stop("cannot set these  values")
+			error("depth<-", "cannot set these  values")
 		}
 		return(x)
 	}
 )
 
+setMethod("linearUnits", signature(x="SpatRaster"), 
+	function(x, ...) {
+		.getLinearUnits(crs(x))
+	}
+)
+
+setMethod("linearUnits", signature(x="SpatVector"), 
+	function(x, ...) {
+		.getLinearUnits(crs(x))
+	}
+)
 
 setMethod("units", signature(x="SpatRaster"), 
 	function(x) { 
@@ -68,8 +83,24 @@ setMethod("units<-", signature(x="SpatRaster"),
 	function(x, value)  {
 		value <- as.character(value)
 		if (! x@ptr$set_units(value)) {
-			stop("cannot set these  values")
+			error("units<-", "cannot set these  values")
 		}
 		return(x)
 	}
 )
+
+
+setMethod("units", signature(x="SpatRasterDataset"), 
+	function(x) { 
+		x@ptr$units
+	}
+)
+
+setMethod("units<-", signature(x="SpatRasterDataset"), 
+	function(x, value)  {
+		value <- as.character(value)
+		x@ptr$units <- value
+		return(x)
+	}
+)
+
