@@ -5,12 +5,11 @@
 #	}
 #)
 
-
 setMethod("vect", signature(x="missing"), 
 	function(...) {
 		p <- methods::new("SpatVector")
 		p@ptr <- SpatVector$new()
-		show_messages(p, "vect")
+		messages(p, "vect")
 		return(p)
 	}
 )
@@ -32,7 +31,7 @@ setMethod("vect", signature(x="character"),
 			x <- normalizePath(x)
 			p@ptr$read(x)
 		}
-		show_messages(p, "vect")
+		messages(p, "vect")
 	}
 )
 
@@ -52,7 +51,7 @@ setMethod("vect", signature(x="sf"),
 .checkXYnames <- function(x) {
 	if (is.null(x)) return(TRUE)
 	if (length(x) != 2) {
-		stop("coordinate matrix should have 2 columns")
+		error("vect", "coordinate matrix should have 2 columns")
 	}
 
 	x <- substr(tolower(x)[1:2], 1, 3)
@@ -61,13 +60,13 @@ setMethod("vect", signature(x="sf"),
 	if ((x[1] == "eas") & (x[2] == "nor")) return(TRUE)
 	if ((x[1] == "lon") & (x[2] == "lat")) return(TRUE)
 	if ((x[1] == "lat") | (x[2] == "lon")) {
-		stop("longitude/latitude in the wrong order")
+		error("vect", "longitude/latitude in the wrong order")
 	} else if ((y[1] == "y") | (y[2] == "x")) {
-		stop("x/y in the wrong order", call. = FALSE)
+		error("vect", "x/y in the wrong order")
 	} else if ((x[1] == "nor") | (x[2] == "eas")) {
-		stop("easting/northing in the wrong order", call. = FALSE)
+		error("vect", "easting/northing in the wrong order")
 	} else {
-		warning("coordinate names not recognized. Expecting lon/lat, x/y, or easting/northing", call. = FALSE)
+		warn("coordinate names not recognized. Expecting lon/lat, x/y, or easting/northing")
 	}
 }
 
@@ -76,7 +75,7 @@ setMethod("vect", signature(x="matrix"),
 		type <- tolower(type)
 		stopifnot(type %in% c("points", "lines", "polygons"))
 		stopifnot(NCOL(x) > 1)
-		
+
 		p <- methods::new("SpatVector")
 		p@ptr <- SpatVector$new()
 		nr <- nrow(x)
@@ -85,20 +84,20 @@ setMethod("vect", signature(x="matrix"),
 		}
 
 		if (ncol(x) == 2) { 
-			.checkXYnames(colnames(x))	
+			.checkXYnames(colnames(x))
 			if (type == "points") {	# treat as unique points
 				p@ptr$setGeometry(type, 1:nr, rep(1, nr), x[,1], x[,2], rep(FALSE, nr))
 			} else {
 				p@ptr$setGeometry(type, rep(1, nr), rep(1, nr), x[,1], x[,2], rep(FALSE, nr))
 			}
 		} else if (ncol(x) == 4) {
-			#.checkXYnames(colnames(x)[3:4])	
-			p@ptr$setGeometry(type, x[,1], x[,2], x[,3], x[,4], rep(FALSE, nr))		
+			#.checkXYnames(colnames(x)[3:4])
+			p@ptr$setGeometry(type, x[,1], x[,2], x[,3], x[,4], rep(FALSE, nr))
 		} else if (ncol(x) == 5) {
-			#.checkXYnames(colnames(x)[3:4])	
+			#.checkXYnames(colnames(x)[3:4])
 			p@ptr$setGeometry(type, x[,1], x[,2], x[,3], x[,4], x[,5])
 		} else {
-			stop("not an appropriate matrix")
+			error("vect", "not an appropriate matrix")
 		}
 		if (!is.null(atts)) {
 			if ((nrow(atts) == nrow(p)) & (ncol(atts) > 0)) {
@@ -106,7 +105,7 @@ setMethod("vect", signature(x="matrix"),
 			}
 		}
 		crs(p) <- ifelse(is.na(crs), "", as.character(crs))
-		show_messages(p, "vect")
+		messages(p, "vect")
 	}
 )
 
@@ -132,15 +131,22 @@ function(x, i, j, ... ,drop=FALSE) {
 
 
 
-setMethod("$<-", "SpatVector",  
-	function(x, name, value) { 
+setReplaceMethod("[[", c("SpatVector", "character", "missing"),
+	function(x, i, j, value) {
+		
 		if (is.null(value)) {
-			if (name %in% names(x)) {
-				x@ptr$remove_column(name)
+			for (name in i) {
+				if (name %in% names(x)) {
+					x@ptr$remove_column(name)
+				}
 			}
 			return(x);
 		}
-
+		if (length(i) > 1) {
+			error("[[<-", "you can only set one variable at a time")
+		}
+	
+		name <- i[1]
 		value <- rep(value, length.out=nrow(x))
 
 		if (name %in% names(x)) {
@@ -149,24 +155,40 @@ setMethod("$<-", "SpatVector",
 			values(x) <- d
 		} else {
 			if (is.integer(value)) {
-				ok <- x@ptr$add_column_long(value, name)	
+				ok <- x@ptr$add_column_long(value, name)
 			} else if (is.numeric(value)) {
-				ok <- x@ptr$add_column_double(value, name)	
+				ok <- x@ptr$add_column_double(value, name)
 			} else {
 				ok <- x@ptr$add_column_string(as.character(value), name)
 			}
 			if (!ok) {
-				stop("cannot set these values")
+				error("[[<-,SpatVector", "cannot set these values")
 			}
 		} 
-		return(x)		
+		x
 	}
 )
 
 
+setMethod("$<-", "SpatVector",  
+	function(x, name, value) {
+		x[[name]] <- value
+		x
+	}
+)
+
+
+
+
+
 setMethod("vect", signature(x="data.frame"), 
-	function(x, type="points", atts=NULL, crs=NA, ...) {
-		x <- as.matrix(x)
-		vect(x, type=type, atts=atts, crs=crs, ...)
+	function(x, geom=c("lon", "lat"), crs=NA, ...) {
+		if (length(geom) == 2) {
+			v <- vect(as.matrix(x[,geom]), crs=crs)
+		} else if (length(geom) == 1) {
+			v <- vect(unlist(x[,geom]), crs=crs)
+		}
+		values(v) <- x
+		v
 	}
 )
