@@ -24,3 +24,6 @@
 	r <- rast(cbind(as.matrix(expand.grid(1:3, 1:4)), 1:12), type = "xyz")
 	expect_equal(class(r)[1], "SpatRaster")
 
+	m <- cbind(x=c(1,1,2,2), y=c(1,2,1,2), z1=1:4, z2=5:8)
+	r <- rast(m, type="xyz")
+	expect_equal(as.vector(values(r)), c(2,4,1,3,6,8,5,7))
