@@ -20,7 +20,7 @@
 #include "recycle.h"
 #include "math_utils.h"
 #include "vecmathfun.h"
-#include "modal.h"
+//#include "modal.h"
 
 
 // need to take care of NAs here. OK for NAN, but not for int types
@@ -168,12 +168,11 @@ SpatRaster SpatRaster::arith(SpatRaster x, std::string oper, SpatOptions &opt) {
 	if (!out.compare_geom(x, false, true)) {
 		return(out);
 	}
-	
+
 	if (!(hasValues() & x.hasValues())) {
 		out.setError("raster has no values"); // or warn and treat as NA?
 		return out;
 	}
- 	if (!out.writeStart(opt)) { return out; }
 	if (!readStart()) {
 		out.setError(getError());
 		return(out);
@@ -182,6 +181,12 @@ SpatRaster SpatRaster::arith(SpatRaster x, std::string oper, SpatOptions &opt) {
 		out.setError(x.getError());
 		return(out);
 	}
+ 	if (!out.writeStart(opt)) {
+		readStop();
+		x.readStop();
+		return out;
+	}
+
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> a = readBlock(out.bs, i);
 		std::vector<double> b = x.readBlock(out.bs, i);
@@ -228,7 +233,7 @@ SpatRaster SpatRaster::arith(double x, std::string oper, bool reverse, SpatOptio
 	if (!smooth_operator(oper)) {
 		out.setError("unknown arith function");
 		return out;
-	}	
+	}
 	if (!hasValues()) {
 		out.setError("raster has no values"); // or warn and treat as NA?
 		return out;
@@ -238,9 +243,12 @@ SpatRaster SpatRaster::arith(double x, std::string oper, bool reverse, SpatOptio
 		out.setError(getError());
 		return(out);
 	}
-	
-  	if (!out.writeStart(opt)) { return out; }
-	
+
+  	if (!out.writeStart(opt)) {
+		readStop();
+		return out;
+	}
+
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> a = readBlock(out.bs, i);
 		if (std::isnan(x)) {
@@ -264,7 +272,7 @@ SpatRaster SpatRaster::arith(double x, std::string oper, bool reverse, SpatOptio
 			}
 		} else if (oper == "^") {
 			if (reverse) {
-				for(double& d : a)  d = std::pow(x, d);				
+				for(double& d : a)  d = std::pow(x, d);			
 			} else {
 				for(double& d : a)  d = std::pow(d, x);
 			}
@@ -290,19 +298,19 @@ SpatRaster SpatRaster::arith(double x, std::string oper, bool reverse, SpatOptio
 			}
 		} else if (oper == "<=") {
 			if (reverse) {
-				for(double& d : a) if (!std::isnan(d)) d = x <= d;				
+				for(double& d : a) if (!std::isnan(d)) d = x <= d;			
 			} else {
 				for(double& d : a) if (!std::isnan(d)) d = d <= x;
 			}
 		} else if (oper == ">") {
 			if (reverse) {
-				for(double& d : a) if (!std::isnan(d)) d = x > d;				
+				for(double& d : a) if (!std::isnan(d)) d = x > d;			
 			} else {
 				for(double& d : a) if (!std::isnan(d)) d = d > x;
 			}
 		} else if (oper == "<") {
 			if (reverse) {
-				for(double& d : a) if (!std::isnan(d)) d = x < d;				
+				for(double& d : a) if (!std::isnan(d)) d = x < d;			
 			} else {
 				for(double& d : a) if (!std::isnan(d)) d = d < x;
 			}
@@ -321,9 +329,9 @@ SpatRaster SpatRaster::arith(double x, std::string oper, bool reverse, SpatOptio
 SpatRaster SpatRaster::arith(std::vector<double> x, std::string oper, bool reverse, SpatOptions &opt) {
 
 	if (x.size() == 1) {
-		return(arith(x[0], oper, reverse, opt));	
+		return(arith(x[0], oper, reverse, opt));
 	}
-	
+
 	SpatRaster out = geometry();
 	if (!smooth_operator(oper)) {
 		out.setError("unknown arith function");
@@ -339,13 +347,16 @@ SpatRaster SpatRaster::arith(std::vector<double> x, std::string oper, bool rever
 		return(out);
 	}
 
-		
-  	if (!out.writeStart(opt)) { return out; }
+	
+  	if (!out.writeStart(opt)) {
+		readStop();
+		return out;
+	}
 
 	unsigned nl = nlyr();
 	unsigned nc = ncol();
 	recycle(x, nlyr());
-	
+
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> v = readBlock(out.bs, i);
 		std::vector<double> vv;
@@ -373,7 +384,7 @@ SpatRaster SpatRaster::arith(std::vector<double> x, std::string oper, bool rever
 				}
 			} else if (oper == "^") {
 				if (reverse) {
-					for(double& d : a)  d = std::pow(x[j], d);				
+					for(double& d : a)  d = std::pow(x[j], d);			
 				} else {
 					for(double& d : a)  d = std::pow(d, x[j]);
 				}
@@ -393,25 +404,25 @@ SpatRaster SpatRaster::arith(std::vector<double> x, std::string oper, bool rever
 				for(double& d : a) if (!std::isnan(d)) d = d != x[j];
 			} else if (oper == ">=") {
 				if (reverse) {
-					for(double& d : a) if (!std::isnan(d)) d = x[j] >= d;				
+					for(double& d : a) if (!std::isnan(d)) d = x[j] >= d;			
 				} else {
 					for(double& d : a) if (!std::isnan(d)) d = d >= x[j];
 				}
 			} else if (oper == "<=") {
 				if (reverse) {
-					for(double& d : a) if (!std::isnan(d)) d = x[j] <= d;				
+					for(double& d : a) if (!std::isnan(d)) d = x[j] <= d;			
 				} else {
 					for(double& d : a) if (!std::isnan(d)) d = d <= x[j];
 				}
 			} else if (oper == ">") {
 				if (reverse) {
-					for(double& d : a) if (!std::isnan(d)) d = x[j] > d;				
+					for(double& d : a) if (!std::isnan(d)) d = x[j] > d;			
 				} else {
 					for(double& d : a) if (!std::isnan(d)) d = d > x[j];
 				}
 			} else if (oper == "<") {
 				if (reverse) {
-					for(double& d : a) if (!std::isnan(d)) d = x[j] < d;				
+					for(double& d : a) if (!std::isnan(d)) d = x[j] < d;			
 				} else {
 					for(double& d : a) if (!std::isnan(d)) d = d < x[j];
 				}
@@ -482,13 +493,16 @@ SpatRaster SpatRaster::math(std::string fun, SpatOptions &opt) {
 		out.setError(getError());
 		return(out);
 	}
+
 	
-		
-  	if (!out.writeStart(opt)) { return out; }
+  	if (!out.writeStart(opt)) {
+		readStop();
+		return out;
+	}
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> a = readBlock(out.bs, i);
 		for(double& d : a) if (!std::isnan(d)) d = mathFun(d);
-		if (!out.writeValues(a, out.bs.row[i], out.bs.nrows[i], 0, ncol())) return out;		
+		if (!out.writeValues(a, out.bs.row[i], out.bs.nrows[i], 0, ncol())) return out;	
 	}
 	out.writeStop();
 	readStop();
@@ -512,9 +526,12 @@ SpatRaster SpatRaster::math2(std::string fun, unsigned digits, SpatOptions &opt)
 		out.setError(getError());
 		return(out);
 	}
+
 	
-		
-  	if (!out.writeStart(opt)) { return out; }
+  	if (!out.writeStart(opt)) {
+		readStop();
+		return out;
+	}
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> a = readBlock(out.bs, i);
 		if (fun == "round") {
@@ -593,9 +610,12 @@ SpatRaster SpatRaster::trig(std::string fun, SpatOptions &opt) {
 		out.setError(getError());
 		return(out);
 	}
+
 	
-		
-  	if (!out.writeStart(opt)) { return out; }
+  	if (!out.writeStart(opt)) {
+		readStop();
+		return out;
+	}
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> a = readValues(out.bs.row[i], out.bs.nrows[i], 0, ncol());
 		for (double& d : a) if (!std::isnan(d)) d = trigFun(d);
@@ -610,17 +630,19 @@ SpatRaster SpatRaster::trig(std::string fun, SpatOptions &opt) {
 SpatRaster SpatRaster::atan_2(SpatRaster x, SpatOptions &opt) {
 	SpatRaster out = geometry();
 	if (!hasValues()) return out;
-  	if (!out.writeStart(opt)) { return out; }
 	if (!readStart()) {
 		out.setError(getError());
 		return(out);
 	}
-	
-		
 	if (!x.readStart()) {
 		out.setError(x.getError());
 		return(out);
-	}	
+	}
+  	if (!out.writeStart(opt)) {
+		readStop();
+		x.readStop();
+		return out;
+	}
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> a = readValues(out.bs.row[i], out.bs.nrows[i], 0, ncol());
 		std::vector<double> b = x.readValues(out.bs.row[i], out.bs.nrows[i], 0, ncol());
@@ -676,27 +698,30 @@ SpatRaster SpatRaster::isnot(SpatOptions &opt) {
 	if (!readStart()) {
 		out.setError(getError());
 		return(out);
-	}	
-  	if (!out.writeStart(opt)) { return out; }
+	}
+  	if (!out.writeStart(opt)) {
+		readStop();
+		return out;
+	}
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> a = readBlock(out.bs, i);
 		for (size_t j=0; j<a.size(); j++) {
 			a[i] = !a[i];
 		}
 		if (!out.writeValues(a, out.bs.row[i], out.bs.nrows[i], 0, ncol())) return out;
-		
+	
 	}
 	out.writeStop();
-	readStop();	
+	readStop();
 	return(out);
 }
 
 
 
 SpatRaster SpatRaster::logic(SpatRaster x, std::string oper, SpatOptions &opt) {
-	
+
 	SpatRaster out = geometry();
-	
+
 	std::vector<std::string> f {"&", "|"}; 
 	if (std::find(f.begin(), f.end(), oper) == f.end()) {
 		out.setError("unknown logic function");
@@ -706,7 +731,7 @@ SpatRaster SpatRaster::logic(SpatRaster x, std::string oper, SpatOptions &opt) {
 	if (!out.compare_geom(x, true, false)) {
 		return(out);
 	}
-	
+
  	if (!readStart()) {
 		out.setError(getError());
 		return(out);
@@ -716,8 +741,10 @@ SpatRaster SpatRaster::logic(SpatRaster x, std::string oper, SpatOptions &opt) {
 		return(out);
 	}
 	
-		
- 	if (!out.writeStart(opt)) { return out; }
+ 	if (!out.writeStart(opt)) {
+		readStop();
+		return out;
+	}
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> a = readBlock(out.bs, i);
 		std::vector<double> b = x.readBlock(out.bs, i);
@@ -729,11 +756,11 @@ SpatRaster SpatRaster::logic(SpatRaster x, std::string oper, SpatOptions &opt) {
 			// stop
 		}
 		if (!out.writeValues(a, out.bs.row[i], out.bs.nrows[i], 0, ncol())) return out;
-		
+	
 	}
 	out.writeStop();
-	readStop();	
-	x.readStop();	
+	readStop();
+	x.readStop();
 	return(out);
 }
 
@@ -742,14 +769,16 @@ SpatRaster SpatRaster::logic(SpatRaster x, std::string oper, SpatOptions &opt) {
 SpatRaster SpatRaster::logic(bool x, std::string oper, SpatOptions &opt) {
 
 	SpatRaster out = geometry();
-	
+
 	if (!readStart()) {
 		out.setError(getError());
 		return(out);
 	}
 	
-		
-  	if (!out.writeStart(opt)) { return out; }
+  	if (!out.writeStart(opt)) {
+		readStop();
+		return out;
+	}
 	std::vector<double> v, m;
 	for (size_t i = 0; i < out.bs.n; i++) {
 		std::vector<double> a = readBlock(out.bs, i);
@@ -766,7 +795,7 @@ SpatRaster SpatRaster::logic(bool x, std::string oper, SpatOptions &opt) {
 		if (!out.writeValues(a, out.bs.row[i], out.bs.nrows[i], 0, ncol())) return out;
 	}
 	out.writeStop();
-	readStop();		
+	readStop();	
 	return(out);
 }
 
@@ -790,8 +819,11 @@ SpatRaster SpatRaster::cum(std::string fun, bool narm, SpatOptions &opt) {
 		return(out);
 	}
 
-	
-  	if (!out.writeStart(opt)) { return out; }
+
+  	if (!out.writeStart(opt)) {
+		readStop();
+		return out;
+	}
 	unsigned nl = out.nlyr();
 	std::vector<double> v(nl);
 	unsigned nc;
@@ -830,7 +862,7 @@ double vstdev(std::vector<double> v, bool narm) {
 	m = vmean(v, narm);
 	return sqrt(m);
 }
-	
+
 
 
 SpatRaster SpatRaster::summary_numb(std::string fun, std::vector<double> add, bool narm, SpatOptions &opt) {
@@ -861,8 +893,11 @@ SpatRaster SpatRaster::summary_numb(std::string fun, std::vector<double> add, bo
 		return(out);
 	}
 
-	
-	if (!out.writeStart(opt)) { return out; }
+
+	if (!out.writeStart(opt)) {
+		readStop();
+		return out;
+	}
 	unsigned nl = nlyr();
 	std::vector<double> v(nl);
 	if (add.size() > 0) v.insert( v.end(), add.begin(), add.end() );
@@ -913,8 +948,11 @@ SpatRaster SpatRaster::modal(std::vector<double> add, std::string ties, bool nar
 		out.setError(getError());
 		return(out);
 	}
-	
-  	if (!out.writeStart(opt)) { return out; }
+
+  	if (!out.writeStart(opt)) {
+		readStop();
+		return out;
+	}
 
 	uint32_t seed = 1;
 	std::default_random_engine rgen(seed);
@@ -931,7 +969,7 @@ SpatRaster SpatRaster::modal(std::vector<double> add, std::string ties, bool nar
 		for (size_t j=0; j<nc; j++) {
 			for (size_t k=0; k<nl; k++) {
 				v[k] = a[j+k*nc];
-			}		
+			}	
 			b[j] = modal_value(v, ities, narm, rgen, dist);
 		}
 		if (!out.writeValues(b, out.bs.row[i], out.bs.nrows[i], 0, ncol())) return out;
@@ -955,8 +993,11 @@ SpatRaster SpatRaster::range(std::vector<double> add, bool narm, SpatOptions &op
 		return(out);
 	}
 
-	
-  	if (!out.writeStart(opt)) { return out; }
+
+  	if (!out.writeStart(opt)) {
+		readStop();
+		return out;
+	}
 	unsigned nl = nlyr();
 	std::vector<double> v(nl);
 	v.insert( v.end(), add.begin(), add.end() );
@@ -1015,8 +1056,11 @@ SpatRaster SpatRasterStack::summary_numb(std::string fun, std::vector<double> ad
 		}
 	}
 
-	
-  	if (!out.writeStart(opt)) { return out; }
+
+  	if (!out.writeStart(opt)) {
+		readStop();
+		return out;
+	}
 	std::vector<double> v(ns);
 	if (add.size() > 0) v.insert( v.end(), add.begin(), add.end() );
 
