@@ -16,12 +16,12 @@
 // along with spat. If not, see <http://www.gnu.org/licenses/>.
 
 // distance
-double distance_plane(double x1, double y1, double x2, double y2);
+double distance_plane(const double &x1, const double &y1, const double &x2, const double &y2);
 std::vector<double> distance_plane(std::vector<double> &x1, std::vector<double> &y1, std::vector<double> &x2, std::vector<double> &y2);
 std::vector<double> distance_plane_vd(std::vector<double> &x1, std::vector<double> &y1, double x2, double y2);
 std::vector<double> distanceToNearest_plane(const std::vector<double> &x1, const std::vector<double> &y1, const std::vector<double> &x2, const std::vector<double> &y2);
 
-double distance_lonlat(double lon1, double lat1, double lon2, double lat2, double a, double f);
+double distance_lonlat(double &lon1, double &lat1, double &lon2, double &lat2, const double &a, const double &f);
 std::vector<double> distance_lonlat(std::vector<double> &lon1, std::vector<double> &lat1, std::vector<double> &lon2, std::vector<double> &lat2, double a, double f) ;
 std::vector<double> distance_lonlat_vd(std::vector<double> &lon1, std::vector<double> &lat1, double lon2, double lat2) ;
 std::vector<double> distanceToNearest_lonlat(const std::vector<double> &lon1, const std::vector<double> &lat1, const std::vector<double> &lon2, const std::vector<double> &lat2, const double &a, const double &f);
