@@ -18,7 +18,7 @@
 #include "spatRaster.h"
 
 bool SpatRaster::readStart() {
-	
+
 	if (!valid_sources(true, true)) {
 		return false;
 	}
@@ -32,7 +32,7 @@ bool SpatRaster::readStart() {
 			source[i].open_read = true;
 		} else if (!readStartGDAL(i)) {
 			return false;
-		}	
+		}
 	}
 	return true;
 }
@@ -44,7 +44,7 @@ bool SpatRaster::readStop() {
 				source[i].open_read = false;
 			} else {
 				readStopGDAL(i); 
-			}	
+			}
 		}
 	}
 	return true;
@@ -64,7 +64,7 @@ std::vector<std::vector<double>> SpatRaster::readBlock2(BlockSize bs, unsigned i
 	size_t off = bs.nrows[i] * ncol();
 	for (size_t i=0; i<nlyr(); i++) {
 		v[i] = std::vector<double>(x.begin()+(i*off), x.begin()+((i+1)*off));
-	}	
+	}
 	return(v);
 }
 
@@ -80,72 +80,67 @@ std::vector<double> SpatRaster::readBlockIP(BlockSize bs, unsigned i) {
 			size_t jj = j * nl + i;
 			v[jj] = lyr[j];
 		}
-	}	
+	}
 	return(v);
 }
 
 
-void SpatRaster::readChunkMEM(std::vector<double> &out, size_t src, uint_64 row, uint_64 nrows, uint_64 col, uint_64 ncols){
+void SpatRaster::readChunkMEM(std::vector<double> &out, size_t src, size_t row, size_t nrows, size_t col, size_t ncols){
 
-	if (hasWindow()) {
-		uint_64 rrow = row + source[0].window.off_row;
-		uint_64 rcol = col + source[0].window.off_col;
-		unsigned endrow = rrow + nrows;
-		unsigned endcol = rcol + ncols;
-		unsigned ncells = source[0].window.full_nrow * source[0].window.full_ncol;
-		unsigned nl = source[src].nlyr;
+	size_t nl = source[src].nlyr;
 
-		if (source[0].window.expanded) {
-			for (size_t lyr=0; lyr < nl; lyr++) {
+	if (source[src].hasWindow) {
+		row += source[src].window.off_row;
+		col += source[src].window.off_col;
+		size_t endrow = row + nrows;
+		size_t endcol = col + ncols;
+		size_t nc = source[src].window.full_ncol;
+		double ncells = source[src].window.full_nrow * nc;
+
+		for (size_t lyr=0; lyr < nl; lyr++) {
+			size_t add = ncells * lyr;
+			for (size_t r = row; r < endrow; r++) {
+				size_t off = add + r * nc;
+				out.insert(out.end(), source[src].values.begin()+off+col, source[src].values.begin()+off+endcol);
+			}
+		}
+			/*
+			else if (source[0].window.expanded) {
 				unsigned add = ncells * lyr;
 				std::vector<double> v1(source[0].window.expand[0] * ncols, NAN);
 				out.insert(out.end(), v1.begin(), v1.end());
 				v1.resize(source[0].window.expand[1], NAN);
 				std::vector<double> v2(source[0].window.expand[2], NAN);
-				for (size_t r = rrow; r < endrow; r++) {
+				for (size_t r = wrow; r < endrow; r++) {
 					unsigned a = add + r * source[0].window.full_ncol;
 					out.insert(out.end(), v1.begin(), v1.end());
-					out.insert(out.end(), source[src].values.begin()+a+rcol, source[src].values.begin()+a+endcol);
+					out.insert(out.end(), source[src].values.begin()+a+wcol, source[src].values.begin()+a+endcol);
 					out.insert(out.end(), v2.begin(), v2.end());
 				}
 				v1.resize(source[0].window.expand[3] * ncols, NAN);
 				out.insert(out.end(), v1.begin(), v1.end());
 			}
-			
-			
-		} else { // !(source[0].window.expanded) 
-
-
-			for (size_t lyr=0; lyr < nl; lyr++) {
-				unsigned add = ncells * lyr;
-				for (size_t r = rrow; r < endrow; r++) {
-					unsigned a = add + r * source[0].window.full_ncol;
-					out.insert(out.end(), source[src].values.begin()+a+rcol, source[src].values.begin()+a+endcol);
-				}
-			}
-		}
-
-	} else { //	!hasWindow()
-
+			*/
+	
+	} else { //	no window
 		if (row==0 && nrows==nrow() && col==0 && ncols==ncol()) {
 			out.insert(out.end(), source[src].values.begin(), source[src].values.end());
 		} else {
-			unsigned nl = source[src].nlyr;
-			unsigned ncells = ncell();
+			double ncells = ncell();
 			if (col==0 && ncols==ncol()) {
 				for (size_t lyr=0; lyr < nl; lyr++) {
-					unsigned add = ncells * lyr;
-					unsigned a = add + row * ncol();
-					unsigned b = a + nrows * ncol();
+					size_t add = ncells * lyr;
+					size_t a = add + row * ncol();
+					size_t b = a + nrows * ncol();
 					out.insert(out.end(), source[src].values.begin()+a, source[src].values.begin()+b);
 				}
 			} else {
-				unsigned endrow = row + nrows;
-				unsigned endcol = col + ncols;
+				size_t endrow = row + nrows;
+				size_t endcol = col + ncols;
 				for (size_t lyr=0; lyr < nl; lyr++) {
-					unsigned add = ncells * lyr;
+					size_t add = ncells * lyr;
 					for (size_t r = row; r < endrow; r++) {
-						unsigned a = add + r * ncol();
+						size_t a = add + r * ncol();
 						out.insert(out.end(), source[src].values.begin()+a+col, source[src].values.begin()+a+endcol);
 					}
 				}
@@ -156,40 +151,37 @@ void SpatRaster::readChunkMEM(std::vector<double> &out, size_t src, uint_64 row,
 
 
 
-
-std::vector<double> SpatRaster::readValues(uint_64 row, uint_64 nrows, uint_64 col, uint_64 ncols){
+std::vector<double> SpatRaster::readValues(size_t row, size_t nrows, size_t col, size_t ncols){
 
 	std::vector<double> out;
 	if (!hasValues()) {
 		//setError
 		return out; // or NAs?
 	}
-	
-	row = std::min(std::max(uint_64(0), row), nrow()-1);
-	col = std::min(std::max(uint_64(0), col), ncol()-1);
-	nrows = std::max(uint_64(1), std::min(nrows, nrow()-row));
-	ncols = std::max(uint_64(1), std::min(ncols, ncol()-col));
+
+	row = std::min(std::max(size_t(0), row), nrow()-1);
+	col = std::min(std::max(size_t(0), col), ncol()-1);
+	nrows = std::max(size_t(1), std::min(nrows, nrow()-row));
+	ncols = std::max(size_t(1), std::min(ncols, ncol()-col));
 	if ((nrows==0) | (ncols==0)) {
 		return out;
 	}
 	unsigned n = nsrc();
-	
+
 	for (size_t src=0; src<n; src++) {
 		if (source[src].memory) {
 			readChunkMEM(out, src, row, nrows, col, ncols);
 		} else {
 			// read from file
 			#ifdef useGDAL
-			if (hasWindow()) {
 
-				if (!source[0].window.expanded) {
-					readChunkGDAL(out, src, source[0].window.off_row, nrows, source[0].window.off_col, ncols);
-				} else {
+/* 
+				if (source[0].window.expanded) {
 					std::vector<double> gout;
 					readChunkGDAL(gout, src, source[0].window.off_row, nrows, source[0].window.off_col, ncols);
-									
-					uint_64 rrow = row + source[0].window.off_row;
-					uint_64 rcol = col + source[0].window.off_col;
+								
+					size_t rrow = row + source[0].window.off_row;
+					size_t rcol = col + source[0].window.off_col;
 					unsigned endrow = rrow + nrows;
 					unsigned endcol = rcol + ncols;
 					unsigned ncells = source[0].window.full_nrow * source[0].window.full_ncol;
@@ -211,9 +203,8 @@ std::vector<double> SpatRaster::readValues(uint_64 row, uint_64 nrows, uint_64 c
 						out.insert(out.end(), v1.begin(), v1.end());
 					}
 				}
-			} else {
-				readChunkGDAL(out, src, row, nrows, col, ncols);
-			}
+				*/
+			readChunkGDAL(out, src, row, nrows, col, ncols);
 			#endif // useGDAL
 		}
 	}
@@ -221,11 +212,47 @@ std::vector<double> SpatRaster::readValues(uint_64 row, uint_64 nrows, uint_64 c
 }
 
 
+
+bool SpatRaster::readAll() {
+	if (!hasValues()) {
+		return true; 
+	}
+
+	size_t row =0, col=0, nrows=nrow(), ncols=ncol();
+	readStart();
+	size_t n = nsrc();
+	for (size_t src=0; src<n; src++) {
+		if (!source[src].memory) {
+			readChunkGDAL(source[src].values, src, row, nrows, col, ncols);
+			source[src].memory = true;
+			source[src].filename = "";
+		}
+		if (src > 0) {
+			if (!source[0].combine_sources(source[src])) {
+				setError("could not combine sources");
+				return false;
+			}
+			source[src].values.resize(0);
+		}	
+	}
+	readStop();
+	if (n>1) source.resize(1);
+	return true;
+}
+
+
 std::vector<double> SpatRaster::getValues(long lyr) {
 
 	std::vector<double> out;
-	
-	if (hasWindow()) {
+
+	bool hw = false;
+	for (size_t i=0; i<source.size(); i++) {
+		if (source[i].hasWindow) {
+			hw = true;
+			break;
+		}
+	}
+	if (hw) {
 		if (!readStart()) return out;
 		if (lyr < 0) { // default; read all
 			out = readValues(0, nrow(), 0, ncol());
@@ -238,7 +265,7 @@ std::vector<double> SpatRaster::getValues(long lyr) {
 		readStop();
 		return out;
 	}
-		
+	
 	if (lyr < 0) { // default; read all
 		unsigned n = nsrc();
 		for (size_t src=0; src<n; src++) {
@@ -268,13 +295,20 @@ std::vector<double> SpatRaster::getValues(long lyr) {
 
 
 bool SpatRaster::getValuesSource(size_t src, std::vector<double> &out) {
-	
+
 	unsigned n = nsrc();
 	if (src > n) {
 		return false;
 	}
 
-	if (hasWindow()) {
+	bool hw = false;
+	for (size_t i=0; i<source.size(); i++) {
+		if (source[i].hasWindow) {
+			hw = true;
+			break;
+		}
+	}
+	if (hw) {
 		SpatRaster sub = SpatRaster(source[src]);
 		if (!readStart()) return false;
 		out = sub.readValues(0, nrow(), 0, ncol());
@@ -289,7 +323,7 @@ bool SpatRaster::getValuesSource(size_t src, std::vector<double> &out) {
 		#ifdef useGDAL
 		out = readValuesGDAL(src, 0, nrow(), 0, ncol());
 		#endif // useGDAL
-	}	
+	}
 	return true;
 }
 
