@@ -31,7 +31,6 @@
 #endif
 
 typedef long long int_64;
-typedef unsigned long long uint_64;
 
 class SpatCategories {
 	public:
@@ -43,15 +42,15 @@ class SpatCategories {
 class SpatWindow {
 	public:
 		SpatExtent full_extent;
-		uint_64 full_ncol, full_nrow, off_row, off_col;
+		size_t full_ncol, full_nrow, off_row, off_col;
 		bool expanded = false;
-		std::vector<uint_64> expand;
+		std::vector<size_t> expand;
 };
 
 
 
 
-class RasterSource {
+class SpatRasterSource {
     private:
 //		std::ofstream ofs;
 	public:
@@ -61,13 +60,13 @@ class RasterSource {
 		bool open_read=false;
 		bool open_write=false;
 
-		RasterSource();
+		SpatRasterSource();
 
 //		void fsopen(std::string filename);
 //		bool fswrite(std::vector<double> &v);
 //		void fsclose();
 
-		uint_64 ncol, nrow;
+		size_t ncol, nrow;
 		unsigned nlyr;
 		unsigned nlyrfile = 0;
 		SpatExtent extent;
@@ -80,8 +79,12 @@ class RasterSource {
 		//std::vector<std::string> crs = std::vector<std::string>(2, "");
 		SpatSRS srs;
 		std::vector<unsigned> layers;
+		// layer names 
 		std::vector<std::string> names;
-		std::vector<std::string> long_names;
+		// data source (sds) has one "variable name" / long_name
+		std::string source_name;
+		std::string source_name_long;
+		
 		std::vector<int_64> time;
 		std::string timestep = "seconds";
 		bool hasTime = false;
@@ -90,8 +93,8 @@ class RasterSource {
 
 		//std::vector< std::vector<double> values;
         std::vector<double> values;
-        std::vector<int64_t> ivalues;
-        std::vector<bool> bvalues;
+        //std::vector<int64_t> ivalues;
+        //std::vector<bool> bvalues;
 
 		std::vector<bool> hasRange;
 		std::vector<double> range_min;
@@ -109,20 +112,21 @@ class RasterSource {
 		std::string driver;
 		std::string datatype; 
 		
-		// for native files
-		//double NAflag;
+		// user set for reading:
+		bool hasNAflag = false;
+		double NAflag = NAN;
 
 		std::vector<bool> has_scale_offset;
 		std::vector<double> scale;
 		std::vector<double> offset;
 
-//		std::vector<RasterSource> subset(std::vector<unsigned> lyrs);
-		RasterSource subset(std::vector<unsigned> lyrs);
+//		std::vector<SpatRasterSource> subset(std::vector<unsigned> lyrs);
+		SpatRasterSource subset(std::vector<unsigned> lyrs);
 		std::vector<double> getValues(unsigned lyr);
 		void setRange();
 		void resize(unsigned n);
 		bool in_order();
-		bool combine_sources(const RasterSource &x);
+		bool combine_sources(const SpatRasterSource &x);
 
 		bool parameters_changed = false;		
 		
@@ -133,13 +137,21 @@ class RasterSource {
 
 class BlockSize {
 	public:
-		std::vector<uint_64> row;
-		std::vector<uint_64> nrows;
+		std::vector<size_t> row;
+		std::vector<size_t> nrows;
 		unsigned n;
 };
 
 class SpatRaster {
 
+    private:
+		std::string copy_driver = "";
+		std::string copy_filename = "";
+		bool compute_stats = true;
+		bool gdal_stats = false;
+		bool gdal_approx = true;
+		bool gdal_minmax = true;
+
 	protected:
 		//SpatExtent extent;
 		SpatExtent window;
@@ -158,9 +170,9 @@ class SpatRaster {
 // properties and property-like methods for entire object
 ////////////////////////////////////////////////////
 
-		std::string name;
+		//std::string name;
 		
-		std::vector<RasterSource> source;
+		std::vector<SpatRasterSource> source;
 
 		BlockSize bs;
 		//BlockSize getBlockSize(unsigned n, double frac, unsigned steps=0);
@@ -169,16 +181,18 @@ class SpatRaster {
 
 		SpatMessages msg;
 		void setError(std::string s) { msg.setError(s); }
-		std::string getError() { return msg.getError(); }
 		void addWarning(std::string s) { msg.addWarning(s); }
 		void setMessage(std::string s) { msg.setMessage(s); }
 		bool hasError() { return msg.has_error; }
 		bool hasWarning() { return msg.has_warning; }
+		std::string getWarnings() { return msg.getWarnings();}
+		std::string getError() { return msg.getError();}
+		std::string getMessage() { return msg.getMessage();}
 
 		//double NA = std::numeric_limits<double>::quiet_NaN();
 
-		uint_64 ncol();
-		uint_64 nrow();
+		size_t ncol();
+		size_t nrow();
 		SpatExtent getExtent();
 		void setExtent(SpatExtent e);
 		void setExtent(SpatExtent ext, bool keepRes=false, std::string snap="");  // also set it for sources?
@@ -191,12 +205,18 @@ class SpatRaster {
 		bool setSRS(std::string crs);
 
 
+
 /*
 #ifdef useGDAL	
 		bool setSRS(OGRSpatialReference *poSRS, std::string &msg) {
 #endif 
 */
 
+		//bool isLonLat();
+		//bool couldBeLonLat();
+		//bool isGlobalLonLat();
+
+		bool is_geographic();
 		bool is_lonlat();
 		bool could_be_lonlat();
 		bool is_global_lonlat();
@@ -211,7 +231,7 @@ class SpatRaster {
 		std::vector<double> origin();
 		unsigned nlyr();
 
-		// only no values allowed with a single RasterSource
+		// only no values allowed with a single SpatRasterSource
 		bool hasValues() { return source[0].hasValues ; };
 		std::vector<double> getValues(long lyr= -1);
 		
@@ -237,10 +257,15 @@ class SpatRaster {
 		std::vector<bool> hasRange();
 		std::vector<double> range_min();
 		std::vector<double> range_max();
+
 		std::vector<std::string> getNames();
-		std::vector<std::string> getLongNames();
-		bool setLongNames(std::vector<std::string> nms);
 		bool setNames(std::vector<std::string> names, bool make_valid=false);
+
+		std::vector<std::string> getSourceNames();
+		bool setSourceNames(std::vector<std::string>);
+		std::vector<std::string> getLongSourceNames();
+		bool setLongSourceNames(std::vector<std::string>);
+
 		bool hasTime();
 		std::vector<int_64> getTime();
 		std::string getTimeStep();
@@ -253,6 +278,10 @@ class SpatRaster {
 		std::vector<std::string> getUnit();
 		bool setUnit(std::vector<std::string> units);
 
+		bool setNAflag(std::vector<double> flag);
+		std::vector<double> getNAflag();
+
+
 ////////////////////////////////////////////////////
 // constructors
 ////////////////////////////////////////////////////
@@ -262,9 +291,9 @@ class SpatRaster {
 		SpatRaster(std::vector<unsigned> rcl, std::vector<double> ext, std::string crs);
 		SpatRaster(std::vector<std::string> fname, std::vector<int> subds, std::vector<std::string> subdsname, std::string x);
 		SpatRaster(std::string fname, std::vector<int> subds, std::vector<std::string> subdsname);
-		SpatRaster(RasterSource s);
-		void setSource(RasterSource s);
-		void setSources(std::vector<RasterSource> s);
+		SpatRaster(SpatRasterSource s);
+		void setSource(SpatRasterSource s);
+		void setSources(std::vector<SpatRasterSource> s);
 		//SpatRaster(const SpatRaster& x);
 
         SpatRaster deepCopy();
@@ -276,10 +305,10 @@ class SpatRaster {
 //		bool constructFromNCDFsds(std::string filename, std::vector<std::string> meta, std::vector<int> subds, std::vector<std::string> subdsname);
 
 
-		void addSource(SpatRaster x);
+		void addSource(SpatRaster x);	
 		SpatRaster combineSources(SpatRaster x);
 		SpatRaster subset(std::vector<unsigned> lyrs, SpatOptions &opt);
-
+		SpatRaster replace(SpatRaster x, unsigned layer, SpatOptions &opt);
 ////////////////////////////////////////////////////
 // helper methods
 ////////////////////////////////////////////////////
@@ -320,9 +349,11 @@ class SpatRaster {
         std::vector<unsigned> lyrsBySource();
         unsigned nsrc();
 
-		void createCategories(unsigned layer);
+		SpatRaster makeCategorical(unsigned layer, SpatOptions opt);
+		bool createCategories(unsigned layer);
 		std::vector<bool> hasCategories();
-		void setCategories(unsigned layer, std::vector<std::string> labs, std::vector<double> levs);
+		bool setCategories(unsigned layer, std::vector<double> levels, std::vector<std::string> labels);
+		bool removeCategories(unsigned layer);
 		std::vector<SpatCategories> getCategories();
 		SpatCategories getLayerCategories(unsigned layer);
 
@@ -333,25 +364,21 @@ class SpatRaster {
 		SpatDataFrame getLayerAttributes(unsigned layer);
 		std::vector<bool> hasColors();
 		std::vector<SpatDataFrame> getColors();
+		bool setColors(size_t layer, SpatDataFrame cols);
 
 		double valuesCell(double);
 		double valuesCell(int, int);
 		std::vector<double> valuesCell(std::vector<double>);
 		std::vector<double> valuesRow(int);
 
-		bool isLonLat();
-		bool couldBeLonLat();
-		bool isGlobalLonLat();
-
-
 ////////////////////////////////////////////////////
 // read and write
 ////////////////////////////////////////////////////
 
 		bool valid_sources(bool files=true, bool rotated=true);
 		bool readStart();
-		std::vector<double> readValues(uint_64 row, uint_64 nrows, uint_64 col, uint_64 ncols);
-		void readChunkMEM(std::vector<double> &out, size_t src, uint_64 row, uint_64 nrows, uint_64 col, uint_64 ncols);
+		std::vector<double> readValues(size_t row, size_t nrows, size_t col, size_t ncols);
+		void readChunkMEM(std::vector<double> &out, size_t src, size_t row, size_t nrows, size_t col, size_t ncols);
 
 		std::vector<double> readBlock(BlockSize bs, unsigned i);
 		std::vector<std::vector<double>> readBlock2(BlockSize bs, unsigned i);
@@ -359,23 +386,28 @@ class SpatRaster {
 		std::vector<double> readExtent(SpatExtent e);
 		bool readStop();
 
+		bool readAll();
 
 		bool writeStart(SpatOptions &opt);
-		bool writeValues(std::vector<double> &vals, uint_64 startrow, uint_64 nrows, uint_64 startcol, uint_64 ncols);
-		bool writeValues2(std::vector<std::vector<double>> &vals, uint_64 startrow, uint_64 nrows, uint_64 startcol, uint_64 ncols);
+		bool writeValues(std::vector<double> &vals, size_t startrow, size_t nrows, size_t startcol, size_t ncols);
+		bool writeValues2(std::vector<std::vector<double>> &vals, size_t startrow, size_t nrows, size_t startcol, size_t ncols);
 		bool writeStop();
 		bool writeHDR(std::string filename);
 
 		//bool writeStartGDAL(std::string filename, std::string driver, std::string datatype, bool overwrite, SpatOptions &opt);
 		bool writeStartGDAL(SpatOptions &opt);		
 		bool fillValuesGDAL(double fillvalue);
-		bool writeValuesGDAL(std::vector<double> &vals, uint_64 startrow, uint_64 nrows, uint_64 startcol, uint_64 ncols);
+		bool writeValuesGDAL(std::vector<double> &vals, size_t startrow, size_t nrows, size_t startcol, size_t ncols);
 		bool writeStopGDAL();
 
+
+		std::vector<double> readmulti(std::string filename, std::string var);
+
+
 		//bool writeStartBinary(std::string filename, std::string datatype, std::string bandorder, bool overwrite);
 		//bool writeValuesBinary(std::vector<double> &vals, unsigned startrow, unsigned nrows, unsigned startcol, unsigned ncols);
 
-		bool writeValuesMem(std::vector<double> &vals, uint_64 startrow, uint_64 nrows, uint_64 startcol, uint_64 ncols);
+		bool writeValuesMem(std::vector<double> &vals, size_t startrow, size_t nrows, size_t startcol, size_t ncols);
 
 		// binary (flat) source
 		//std::vector<double> readValuesBinary(unsigned src, unsigned row, unsigned nrows, unsigned col, unsigned ncols);
@@ -383,26 +415,27 @@ class SpatRaster {
 		//std::vector<std::vector<double>> readCellsBinary(unsigned src, std::vector<double> cells);
 
 		// gdal source
-		std::vector<double> readValuesGDAL(unsigned src, uint_64 row, uint_64 nrows, uint_64 col, uint_64 ncols, int lyr = -1);
-		std::vector<double> readGDALsample(unsigned src, int srows, int scols);
+		std::vector<double> readValuesGDAL(unsigned src, size_t row, size_t nrows, size_t col, size_t ncols, int lyr = -1);
+		std::vector<double> readGDALsample(unsigned src, size_t srows, size_t scols);
 		std::vector<std::vector<double>> readRowColGDAL(unsigned src, std::vector<int_64> &rows, const std::vector<int_64> &cols);
 
 		bool readStartGDAL(unsigned src);
 		bool readStopGDAL(unsigned src);
-		void readChunkGDAL(std::vector<double> &data, unsigned src, uint_64 row, unsigned nrows, uint_64 col, unsigned ncols);
+		void readChunkGDAL(std::vector<double> &data, unsigned src, size_t row, unsigned nrows, size_t col, unsigned ncols);
 
 		bool setWindow(SpatExtent x);
 		bool removeWindow();
-		bool hasWindow();
+		std::vector<bool> hasWindow();
 
 		void openFS(std::string const &filename);
 
 		SpatRaster writeRaster(SpatOptions &opt);
 		//SpatRaster writeRasterGDAL(std::string filename, std::string format, std::string datatype, bool overwrite, SpatOptions &opt);
 		//SpatRaster writeRasterBinary(std::string filename, std::string datatype, std::string bandorder, bool overwrite);
+		bool checkFormatRequirements(const std::string &driver, std::string &filename, std::string &datatype);
 
 		bool canProcessInMemory(SpatOptions &opt);
-		uint_64 chunkSize(unsigned n, double frac);
+		size_t chunkSize(unsigned n, double frac);
 
 		void fill(double x);
 
@@ -422,8 +455,8 @@ class SpatRaster {
         std::vector<std::vector<double>> adjacent(std::vector<double> cells, std::string directions, bool include);
  		SpatRaster aggregate(std::vector<unsigned> fact, std::string fun, bool narm, SpatOptions &opt);
 		SpatExtent align(SpatExtent e, std::string snap);
-		SpatRaster rst_area(SpatOptions &opt);
-		std::vector<double> sum_area(SpatOptions &opt);
+		SpatRaster rst_area(bool adjust, SpatOptions &opt);
+		std::vector<double> sum_area(bool adjust, SpatOptions &opt);
 		std::vector<std::vector<double>> area_by_value(SpatOptions &opt);
 
 		SpatRaster arith(SpatRaster x, std::string oper, SpatOptions &opt);
@@ -443,7 +476,8 @@ class SpatRaster {
 
 		SpatRaster buffer(double d, SpatOptions &opt);
 		SpatRaster clamp(double low, double high, bool usevalue, SpatOptions &opt);
-		SpatRaster cover(SpatRaster x, double value, SpatOptions &opt);
+		SpatRaster cover(SpatRaster x, std::vector<double> value, SpatOptions &opt);
+
 		SpatRaster crop(SpatExtent e, std::string snap, SpatOptions &opt);
 		SpatRaster cum(std::string fun, bool narm, SpatOptions &opt);
         SpatRaster disaggregate(std::vector<unsigned> fact, SpatOptions &opt);
@@ -452,11 +486,12 @@ class SpatRaster {
 
 		SpatRaster edges(bool classes, std::string type, unsigned directions, SpatOptions &opt);
 		SpatRaster extend(SpatExtent e, SpatOptions &opt);
-		std::vector<std::vector<std::vector<double>>> extractVector(SpatVector v, bool touches, std::string method="");
-		std::vector<double> getCells(SpatVector v, bool touches, std::string method);
+		std::vector<std::vector<std::vector<double>>> extractVector(SpatVector v, bool touches, std::string method="", bool cells=false, bool weights=false);
+		std::vector<double> vectCells(SpatVector v, bool touches, std::string method, bool weights);
+		std::vector<double> extCells(SpatExtent ext);
 
 		std::vector<std::vector<double>> extractCell(std::vector<double> &cell);
-        std::vector<std::vector<double>> extractXY(std::vector<double> &x, std::vector<double> &y, std::string method);
+        std::vector<std::vector<double>> extractXY(std::vector<double> &x, std::vector<double> &y, std::string method, bool cells=false);
 		
 		SpatRaster flip(bool vertical, SpatOptions &opt);
 		SpatRaster filler(SpatRaster x, SpatOptions &opt);
@@ -491,6 +526,9 @@ class SpatRaster {
 		SpatRaster logic(SpatRaster x, std::string oper, SpatOptions &opt);
 		SpatRaster logic(bool x, std::string oper, SpatOptions &opt);
 		SpatRaster mask(SpatRaster x, bool inverse, double maskvalue, double updatevalue, SpatOptions &opt);
+		SpatRaster mask(SpatRaster x, bool inverse, std::vector<double> maskvalues, double updatevalue, SpatOptions &opt);
+
+
 		SpatRaster mask(SpatVector x, bool inverse, double updatevalue, SpatOptions &opt);
 		SpatRaster math(std::string fun, SpatOptions &opt);
 		SpatRaster math2(std::string fun, unsigned digits, SpatOptions &opt);
@@ -507,22 +545,25 @@ class SpatRaster {
 
 		SpatRaster range(std::vector<double> add, bool narm, SpatOptions &opt);
 		//SpatRaster rasterize(SpatVector p, std::vector<double> values, double background, bool update, SpatOptions &opt);
-		SpatRaster rasterize(SpatVector x, std::string field, std::vector<double> values, double background, bool update, bool touches, bool inverse, SpatOptions &opt);
+		SpatRaster rasterize(SpatVector x, std::string field, std::vector<double> values, std::vector<std::string> labels, double background, bool update, bool touches, bool inverse, SpatOptions &opt);
 		std::vector<double> rasterizeCells(SpatVector &v, bool touches);
+		std::vector<std::vector<double>> rasterizeCellsWeights(SpatVector &v, bool touches);
 
 		SpatRaster reclassify(std::vector<std::vector<double>> rcl, unsigned right, bool lowest, bool othersNA, SpatOptions &opt);
 		SpatRaster reclassify(std::vector<double> rcl, unsigned nc, unsigned right, bool lowest, bool othersNA, SpatOptions &opt);
-		SpatRaster classify_layers(std::vector<std::vector<double>> groups, std::vector<double> id, SpatOptions &opt);
-		SpatRaster classify_layers(std::vector<double> groups, unsigned nc, std::vector<double> id, SpatOptions &opt);
+		//SpatRaster classify_layers(std::vector<std::vector<double>> groups, std::vector<double> id, SpatOptions &opt);
+		//SpatRaster classify_layers(std::vector<double> groups, unsigned nc, std::vector<double> id, SpatOptions &opt);
 
-		std::vector<double> readSample(unsigned src, unsigned srows, unsigned scols);
+		std::vector<double> readSample(unsigned src, size_t srows, size_t scols);
 		SpatRaster rotate(bool left, SpatOptions &opt);
 
+		std::vector<size_t> sampleCells(unsigned size, std::string method, bool replace, unsigned seed);
 		SpatRaster sampleRegularRaster(unsigned size);
 		SpatRaster sampleRandomRaster(unsigned size, bool replace, unsigned seed);
 		std::vector<std::vector<double>> sampleRegularValues(unsigned size);
 		std::vector<std::vector<double>> sampleRandomValues(unsigned size, bool replace, unsigned seed);
 
+		SpatRaster scale(std::vector<double> center, bool docenter, std::vector<double> scale, bool doscale, SpatOptions &opt);
 		SpatRaster slope(unsigned neighbors, bool degrees, SpatOptions &opt);
 
 		SpatRaster selRange(SpatRaster x, int z, int recycleby, SpatOptions &opt);
@@ -539,8 +580,8 @@ class SpatRaster {
 		SpatRaster project2(SpatRaster &x, std::string method, SpatOptions &opt);
 		void project3(SpatRaster &out, std::string method, SpatOptions &opt);
 
-		SpatRaster resample1(SpatRaster &x, const std::string &method, SpatOptions &opt);
-		void resample2(SpatRaster &out, const std::string &method, SpatOptions &opt);
+//		SpatRaster resample1(SpatRaster &x, const std::string &method, SpatOptions &opt);
+//		void resample2(SpatRaster &out, const std::string &method, SpatOptions &opt);
 
 #ifdef useGDAL
 		bool open_gdal(GDALDatasetH &hDS, int src, SpatOptions &opt);
